@@ -1,26 +1,20 @@
-use ansi_term::Style;
+use crossterm::{cursor::MoveTo, event, terminal, ExecutableCommand};
 use dirs::home_dir;
 use serde::{Deserialize, Serialize};
 use text_io::read;
-use tokio::{join, spawn};
-
-use crossterm::{
-    cursor::{MoveLeft, MoveTo},
-    terminal, ExecutableCommand,
-};
 
 use egg_mode::{
     auth::{access_token, authorize_url, request_token},
     error::{Error::TwitterError, TwitterErrors},
-    tweet::{like, mentions_timeline, DraftTweet, Tweet},
+    tweet::{like, mentions_timeline, Tweet},
     KeyPair, Token,
 };
 
 use std::{
     clone::Clone,
     fs,
-    future::Future,
     io::{stdout, Stdout, Write},
+    iter::Iterator,
     process::exit,
 };
 
@@ -43,6 +37,8 @@ struct Keys {
 
 type TwitterResult = Result<(), TwitterErrors>;
 
+const MENU_TEXT: &str = "(l)ike | (n)ext | e(x)it";
+
 async fn authenticate() -> Token {
     let key_file = home_dir().unwrap().join(".config/qt/keys.toml");
     let key_map: Keys = toml::from_str(&fs::read_to_string(&key_file).unwrap()).unwrap();
@@ -102,30 +98,25 @@ async fn send_like(tweet: &Tweet, token: &Token) -> TwitterResult {
     }
 }
 
-async fn send_reply(tweet: &Tweet, reply_text: String, token: &Token) {
-    // TODO should mention everyone in the thread, not just who was directly replied to
-    let reply_text = format!(
-        "@{} {}",
-        tweet.user.as_ref().unwrap().screen_name,
-        reply_text
-    );
-    DraftTweet::new(reply_text)
-        .in_reply_to(tweet.id)
-        .send(&token)
-        .await
-        .unwrap();
+// async fn send_reply(tweet: &Tweet, reply_text: String, token: &Token) {
+//     // TODO should mention everyone in the thread, not just who was directly replied to
+//     let reply_text = format!(
+//         "@{} {}",
+//         tweet.user.as_ref().unwrap().screen_name,
+//         reply_text
+//     );
+//     DraftTweet::new(reply_text)
+//         .in_reply_to(tweet.id)
+//         .send(&token)
+//         .await
+//         .unwrap();
+// }
+
+fn err_view(errs: TwitterErrors) -> String {
+    errs.errors[0].message.clone()
 }
 
-fn show_err(errs: TwitterErrors) {
-    stdout().execute(MoveLeft(2)).unwrap();
-    for err in errs.errors {
-        println!("{}", Style::new().italic().paint(err.message))
-    }
-    print!("\n> ");
-    stdout().flush().unwrap();
-}
-
-fn show_tweet<'a>(tweet: &'a Tweet) -> Vec<Spans<'a>> {
+fn tweet_view<'a>(tweet: &'a Tweet) -> Vec<Spans<'a>> {
     let screen_name = &tweet.user.as_ref().unwrap().screen_name;
     let screen_name_msg = format!("@{} says:\n", screen_name);
     let in_response_msg = match &tweet.in_reply_to_screen_name {
@@ -146,48 +137,61 @@ fn show_tweet<'a>(tweet: &'a Tweet) -> Vec<Spans<'a>> {
     ]
 }
 
-async fn process_actions(tweet: &Tweet, actions: String, token: &Token) -> bool {
-    let mut to_like = false;
-    for action in actions.chars() {
-        if action == 'l' {
-            to_like = true;
-        } else if action == 'r' {
-            let reply_prompt = Style::new().italic().paint("Your reply: ");
-            print!("{}", reply_prompt);
-            stdout().flush().unwrap();
-
-            let reply_text = read!();
-            send_reply(&tweet, reply_text, &token).await;
-            println!();
-        } else if action == 'x' {
-            exit(0);
+fn get_key() -> char {
+    loop {
+        match event::read().unwrap() {
+            event::Event::Key(event) => match event.code {
+                event::KeyCode::Char(c) => return c,
+                _ => (),
+            },
+            _ => (),
         }
     }
-
-    to_like
 }
 
-async fn run_action(action: impl Future<Output = TwitterResult>) {
-    match action.await {
-        Ok(()) => (),
-        Err(e) => show_err(e),
+async fn run_action(tweet: &Tweet, key: char, token: &Token) -> String {
+    if key == 'x' {
+        stdout()
+            .execute(terminal::Clear(terminal::ClearType::All))
+            .unwrap()
+            .execute(MoveTo(0, 0))
+            .unwrap();
+        exit(0);
     };
+    if key == 'l' {
+        match send_like(tweet, token).await {
+            Ok(()) => "Success".to_string(),
+            Err(e) => err_view(e),
+        }
+    } else {
+        "You can't do that".to_string()
+    }
 }
 
 fn draw(text_0: Vec<Spans>, text_1: Vec<Spans>, terminal: &mut Terminal<CrosstermBackend<Stdout>>) {
     terminal
         .draw(|f| {
             let layout = Layout::default()
-                .constraints([Constraint::Min(5), Constraint::Length(3)])
+                .constraints([
+                    Constraint::Length(3),
+                    Constraint::Min(5),
+                    Constraint::Length(3),
+                ])
                 .split(f.size());
+
             f.render_widget(
-                Paragraph::new(text_0).block(Block::default().borders(Borders::ALL)),
+                Paragraph::new(vec![Spans::from(MENU_TEXT)])
+                    .block(Block::default().borders(Borders::ALL)),
                 layout[0],
             );
             f.render_widget(
-                Paragraph::new(text_1).block(Block::default().borders(Borders::ALL)),
+                Paragraph::new(text_0).block(Block::default().borders(Borders::ALL)),
                 layout[1],
             );
+            f.render_widget(
+                Paragraph::new(text_1).block(Block::default().borders(Borders::ALL)),
+                layout[2],
+            );
         })
         .unwrap();
     let height = terminal::size().unwrap().1;
@@ -213,24 +217,27 @@ async fn main() {
     );
     let tweets = get_mentions(&token).await;
 
-    let mut actions;
-    let mut to_like;
-    let mut next_tweet_to_like = None;
-
+    terminal::enable_raw_mode().unwrap();
     for tweet in tweets {
-        draw(show_tweet(&tweet), vec![Spans::from(">")], &mut terminal);
-        let prompt = spawn(async { read!("{}\n") });
-
-        if next_tweet_to_like.is_some() {
-            let tweet_to_like = next_tweet_to_like.unwrap();
-            actions = join!(prompt, run_action(send_like(&tweet_to_like, &token)))
-                .0
-                .unwrap();
-        } else {
-            actions = prompt.await.unwrap();
+        draw(tweet_view(&tweet), vec![Spans::from("")], &mut terminal);
+        loop {
+            let key = get_key();
+            let message = match key {
+                'n' => break,
+                c => {
+                    draw(
+                        tweet_view(&tweet),
+                        vec![Spans::from("Working...")],
+                        &mut terminal,
+                    );
+                    run_action(&tweet, c, &token).await
+                }
+            };
+            draw(
+                tweet_view(&tweet),
+                vec![Spans::from(message)],
+                &mut terminal,
+            );
         }
-
-        to_like = process_actions(&tweet, actions, &token).await;
-        next_tweet_to_like = if to_like { Some(tweet) } else { None };
     }
 }
